"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const perf_hooks_1 = require("perf_hooks");
const errors_1 = require("./errors");
const newrelic_1 = require("newrelic");
exports.sealed = (constructor) => {
    Object.seal(constructor);
    Object.seal(constructor.prototype);
};
exports.callableOnce = (target, name, descriptor) => {
    const newDescriptor = Object.assign({}, descriptor);
    newDescriptor.value = function () {
        if (!this[`___callableOnce__${name}___`]) {
            descriptor.value.apply(this, arguments);
            this[`___callableOnce__${name}___`] = true;
        }
        else {
            throw new errors_1.PuzzleError(errors_1.ERROR_CODES.CALLABLE_ONCE_CALLED_MORE_THAN_ONE_TIME, target.constructor.name, name);
        }
    };
    return newDescriptor;
};
exports.benchmark = (enabled, logger) => {
    return (target, propertyKey, descriptor) => {
        if (!enabled)
            return descriptor;
        console.log(`Attaching benchmark to ${propertyKey}`);
        const newDescriptor = Object.assign({}, descriptor);
        newDescriptor.value = function () {
            const old_time = perf_hooks_1.performance.now();
            const m0 = process.memoryUsage();
            const c0 = process.cpuUsage();
            const returnValue = descriptor.value.apply(this, arguments);
            const new_time = perf_hooks_1.performance.now();
            const m1 = process.memoryUsage();
            const diffCPU = process.cpuUsage(c0);
            logger({
                benchmarking: `${this.constructor.name} - ${propertyKey}`,
                ram: `${(m1.rss - m0.rss) / 1048576} mb`,
                heapTotal: `${(m1.heapTotal - m0.heapTotal) / 1048576} mb`,
                heapUsed: `${(m1.heapUsed - m0.heapUsed) / 1048576} mb`,
                cpu: `${(diffCPU.user + diffCPU.system) / 1000000} s`,
                spendTime: `${(new_time - old_time)} ms`
            });
            return returnValue;
        };
        return newDescriptor;
    };
};
exports.nrSegment = (segmentName, record = true) => {
    return (target, propertyKey, descriptor) => {
        const original = descriptor.value;
        descriptor.value = function (...args) {
            return newrelic_1.startSegment(segmentName, record, () => {
                return original.apply(this, args);
            });
        };
        return descriptor;
    };
};
exports.nrSegmentAsync = (segmentName, record = true) => {
    return (target, propertyKey, descriptor) => {
        const original = descriptor.value;
        descriptor.value = function (...args) {
            return __awaiter(this, void 0, void 0, function* () {
                return yield newrelic_1.startSegment(segmentName, record, () => __awaiter(this, void 0, void 0, function* () {
                    return yield original.apply(this, args);
                }));
            });
        };
        return descriptor;
    };
};
//# sourceMappingURL=decorators.js.map